<?php
	# Mantis - a php based bugtracking system
	# Copyright (C) 2000 - 2002  Kenzaburo Ito - kenito@300baud.org
	# Copyright (C) 2002 - 2003  Mantis Team   - mantisbt-dev@lists.sourceforge.net
	# This program is distributed under the terms and conditions of the GPL
	# See the README and LICENSE files for details

	# --------------------------------------------------------
	# $Id: group_api.php,v 1.2 2003/08/08 20:16:05 dwoods Exp $
	# --------------------------------------------------------

	$t_core_dir = dirname( __FILE__ ).DIRECTORY_SEPARATOR;
	
	require_once( $t_core_dir . 'email_api.php' );
	require_once( $t_core_dir . 'ldap_api.php' );

	###########################################################################
	# Group API
	###########################################################################

	#===================================
	# Caching
	#===================================

	#########################################
	# SECURITY NOTE: cache globals are initialized here to prevent them
	#   being spoofed if register_globals is turned on
	#
	$g_cache_group = array();

	# --------------------
	# Cache a group row if necessary and return the cached copy
	#  If the second parameter is true (default), trigger an error
	#  if the group can't be found.  If the second parameter is
	#  false, return false if the group can't be found.
	function group_cache_row( $p_group_id, $p_trigger_errors=true) {
		global $g_cache_group;

		$c_group_id = db_prepare_int( $p_group_id );

		$t_group_table = config_get( 'mantis_group_table' );

		if ( isset ( $g_cache_group[$c_group_id] ) ) {
			return $g_cache_group[$c_group_id];
		}

		$query = "SELECT *
				  FROM $t_group_table
				  WHERE id='$c_group_id'";
		$result = db_query( $query );

		if ( 0 == db_num_rows( $result ) ) {
			if ( $p_trigger_errors ) {
				trigger_error( ERROR_GROUP_NOT_FOUND, ERROR );
			} else {
				return false;
			}
		}

		$row = db_fetch_array( $result );

		$g_cache_group[$c_group_id] = $row;

		return $row;
	}

	# --------------------
	# Clear the group cache (or just the given id if specified)
	function group_clear_cache( $p_group_id = null ) {
		global $g_cache_group;

		if ( null === $p_group_id ) {
			$g_cache_group = array();
		} else {
			$c_group_id = db_prepare_int( $p_group_id );
			unset( $g_cache_group[$c_group_id] );
		}

		return true;
	}

	#===================================
	# Boolean queries and ensures
	#===================================

	# --------------------
	# check to see if group exists by id
	# return true if it does, false otherwise
	#
	# Use group_cache_row() to benefit from caching if called multiple times
	#  and because if the user does exist the data may well be wanted
	function group_exists( $p_group_id ) {
		$row = group_cache_row( $p_group_id, false );

		if ( false === $row ) {
			return false;
		} else {
			return true;
		}
	}

	# --------------------
	# check to see if group exists by id
	# if it doesn't exist then error
	#  otherwise let execution continue undisturbed
	function group_ensure_exists( $p_group_id ) {
		if ( ! group_exists( $p_group_id ) ) {
			trigger_error( ERROR_USER_NOT_FOUND, ERROR );
		}
	}

	# --------------------
	# return true if the groupname is unique, false if there is already a group
	#  with that groupname
	function group_is_name_unique( $p_groupname ) {
		$c_groupname = db_prepare_string( $p_groupname );

		$t_group_table = config_get( 'mantis_group_table' );

		$query = "SELECT COUNT(*)
				  FROM $t_group_table
				  WHERE groupname='$c_groupname'";

	    $result = db_query( $query );

	    if ( db_result( $result ) > 0 ) {
			return false;
		} else {
			return true;
		}
	}

	# --------------------
	# Check if the groupname is unique and trigger an ERROR if it isn't
	function group_ensure_name_unique( $p_groupname ) {
		if ( ! group_is_name_unique( $p_groupname ) ) {
			trigger_error( ERROR_USER_NAME_NOT_UNIQUE, ERROR );
		}
	}

	# --------------------
	# Check if the groupname is a valid username (does not account for uniqueness)
	# Return true if it is, false otherwise
	function group_is_name_valid( $p_groupname ) {
		# The DB field is only 32 characters
		if ( strlen( $p_groupname ) > 32 ) {
			return false;	
		}

		# Only allow a basic set of characters
		if ( 0 == preg_match( '/^\w+$/', $p_groupname ) ) {
			return false;
		}

		# We have a valid groupname
		return true;
	}

	# --------------------
	# Check if the groupname is a valid groupname (does not account for uniqueness)
	# Trigger an error if the username is not valid
	function group_ensure_name_valid( $p_groupname ) {
		if ( ! group_is_name_valid( $p_groupname ) ) {
			trigger_error( ERROR_USER_NAME_INVALID, ERROR );
		}
	}

	# --------------------
	# return whether user is monitoring bug for the user id and bug id
	# -- NOT IMPLEMENTED YET --- dwoods
	/*
	function group_is_monitoring_bug( $p_group_id, $p_bug_id ) {
		$c_group_id	= db_prepare_int( $p_group_id );
		$c_bug_id	= db_prepare_int( $p_bug_id );

		$t_bug_monitor_table = config_get( 'mantis_bug_monitor_table' );

		$query = "SELECT COUNT(*)
				  FROM $t_bug_monitor_table
				  WHERE user_id='$c_group_id' AND bug_id='$c_bug_id'";

		$result = db_query( $query );

		if ( 0 == db_result( $result ) ) {
			return false;
		} else {
			return true;
		}
	}
	*/

	# --------------------
	# return true if the user has access of ADMINISTRATOR or higher, false otherwise
	# --- NOT IMPLEMENTED YET --- dwoods
	/*
	function group_is_development( $p_group_id ) {
		$t_access_level = user_get_field( $p_group_id, 'access_level' );

		if ( $t_access_level >= DEVELOPER ) {
			return true;
		} else {
			return false;
		}
	}
	*/

	# --------------------
	# return true is the group account is protected, false otherwise
	function group_is_protected( $p_group_id ) {
		return ( ON == group_get_field( $p_group_id, 'protected' ) );
	}

	# --------------------
	# Trigger an ERROR if the group is protected
	function group_ensure_unprotected( $p_group_id ) {
		if ( group_is_protected( $p_group_id ) ) {
			trigger_error( ERROR_PROTECTED_ACCOUNT, ERROR );
		}
	}

	# --------------------
	# return true is the group is enabled, false otherwise
	function group_is_enabled( $p_group_id ) {
		return ( ON == group_get_field( $p_group_id, 'enabled' ) );
	}

	#===================================
	# Creation / Deletion / Updating
	#===================================

	# --------------------
	# Create a user.
	# returns false if error, the generated cookie string if ok
	function group_create( $p_groupname, $p_access_level=null, $p_protected=false, $p_enabled=true ) {
		if ( null === $p_access_level ) {
			$p_access_level = config_get( 'default_new_group_access_level');
		}

		$c_groupname	= db_prepare_string( $p_groupname );
		$c_access_level	= db_prepare_int( $p_access_level );
		$c_protected	= db_prepare_bool( $p_protected );
		$c_enabled	= db_prepare_bool( $p_enabled );

		group_ensure_name_valid( $p_groupname );
		group_ensure_name_unique( $p_groupname );

		$t_seed = $p_username;
		$t_cookie_string	= auth_generate_unique_cookie_string( $t_seed );

		$t_group_table 		= config_get( 'mantis_group_table' );

		$query = "INSERT INTO $t_group_table
				    ( id, groupname, date_created, enabled, protected, 
				      access_level, cookie_string )
				  VALUES
				    ( null, '$c_groupname', NOW(), $c_enabled, $c_protected, 
				      $c_access_level, '$t_cookie_string')";
		db_query( $query );

		return $t_cookie_string;
	}

	# --------------------
	# delete agroup
	# returns true when the account was successfully deleted
	function group_delete( $p_group_id ) {
		$c_group_id = db_prepare_int($p_group_id);

		group_ensure_unprotected( $p_group_id );

		$t_group_table 			= config_get('mantis_group_table');
		$t_group_user_list_table 	= config_get('mantis_group_user_list_table');

		# Remove account
		$query = "DELETE
				  FROM $t_group_table
				  WHERE id='$c_group_id'";
		db_query( $query );

		$query = "DELETE
				  FROM $t_group_user_list_table
				  WHERE group_id='$c_group_id'";
		db_query( $query );

		group_clear_cache( $p_group_id );

		return true;
    }

	#===================================
	# Data Access
	#===================================

	# --------------------
	# get a group id from a groupname
	#  return false if the groupname does not exist
	function group_get_id_by_name( $p_groupname ) {
		$c_groupname = db_prepare_string( $p_groupname );

		$t_group_table = config_get( 'mantis_group_table' );

		$query = "SELECT id
				  FROM $t_group_table
				  WHERE groupname='$c_groupname'";
		$result = db_query( $query );

		if ( 0 == db_num_rows( $result ) ) {
			return false;
		} else {
			return db_result( $result );
		}
	}

	# --------------------
	# return all data associated with a particular group name
	#  return false if the groupname does not exist
	function group_get_row_by_name( $p_groupname ) {
		$t_group_id = group_get_id_by_name( $p_groupname );

		if ( false === $t_group_id ) {
			return false;
		}

		$row = group_get_row( $t_group_id );

		return $row;
	}

	# --------------------
	# return a group row
	function group_get_row( $p_group_id ) {
		return group_cache_row( $p_group_id );
	}

	# --------------------
	# return the specified group field for the group id
	function group_get_field( $p_group_id, $p_field_name ) {
		if ( NO_GROUP == $p_group_id ) {
			trigger_error( 'group_get_field() for NO_GROUP', WARNING );
			return "@null@";
		}

		$row = group_get_row( $p_group_id );

		if ( isset( $row[$p_field_name] ) ) {
			return $row[$p_field_name];
		} else {
			trigger_error( ERROR_DB_FIELD_NOT_FOUND, WARNING );
			return '';
		}
	}

	# --------------------
	# return the groupname or a string saying "group no longer exists"
	#  if the group does not exist
	function group_get_name( $p_group_id ) {
		$row = group_cache_row( $p_group_id, false );

		if ( false == $row ) {
			return lang_get( 'prefix_for_deleted_groups' ) . (integer)$p_group_id;
		} else {
			return $row['groupname'];
		}
	}

	# --------------------
	# return the groups's access level
	function group_get_access_level( $p_group_id ) {
		$t_access_level  = group_get_field( $p_group_id, 'access_level' );

		return $t_access_level;
	}
	
	# --------------------
	# check to see if the user/group combo already exists
	# returns true is duplicate is found, otherwise false
	function group_includes_user( $p_group_id, $p_user_id ) {
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$c_group_id	= db_prepare_int( $p_group_id );
		$c_user_id	= db_prepare_int( $p_user_id );

		$query = "SELECT COUNT(*)
				  FROM $t_group_user_list_table
				  WHERE group_id='$c_group_id' AND
						user_id='$c_user_id'";
		$result = db_query( $query );

		if ( 0 == db_result( $result ) ) {
			return false;
		} else {
			return true;
		}
	}
	
	# --------------------
	# Return an array of info about users who are part of the given group
	# For each user we have 'id', 'username', and 'access_level' (overall access level)
	# If the second parameter is given, return only users with an access level
	#  higher than the given value.
	function group_get_all_user_rows( $p_group_id, $p_access_level=ANYBODY ) {
		$c_group_id	= db_prepare_int( $p_group_id );

		# Optimization when access_level is NOBODY
		if ( NOBODY == $p_access_level ) {
			return array();
		}

		$t_user_table = config_get( 'mantis_user_table' );
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$t_on = ON;

		$t_access_level = $p_access_level;

		$t_access_clause = '';

		if ( $t_access_level > 0 ) {
			$t_access_clause = ' AND access_level >= ' . $t_access_level;
		}

		$t_users = array();

		$query = "SELECT id, username, access_level
					FROM $t_user_table
					WHERE enabled = $t_on
					  $t_access_clause
					ORDER BY username";

		$result = db_query( $query );
		$t_row_count = db_num_rows( $result );
		for ( $i=0 ; $i < $t_row_count ; $i++ ) {
			$row = db_fetch_array( $result );
			$t_users[$row['id']] = $row;
		}

		$query = "SELECT u.id, u.username
					FROM $t_group_user_list_table l, $t_user_table u
					WHERE l.user_id = u.id
					  AND u.enabled = $t_on
					  AND l.group_id = $c_group_id
					ORDER BY u.username";

		$result = db_query( $query );
		$t_row_count = db_num_rows( $result );
		for ( $i=0 ; $i < $t_row_count ; $i++ ) {
			$row = db_fetch_array( $result );
			$f_access_level = $t_users[$row['id']]['access_level'];
			if ( $f_access_level >= $p_access_level ) {
				$t_users[$row['id']] = $row;
				$t_users[$row['id']]['access_level'] = $f_access_level;
			} else {
				# If user's overridden level is lower than required, so remove
				#  them from the list if they were previously there
				unset( $t_users[$row['id']] );
			}
		}

		return multi_sort( array_values($t_users), 'username' );
	}
	
	# --------------------
	# Return an array of user_id's that belong to the given group, if their access
	# level is above the access given.
	function group_get_users( $p_group_id, $p_access_level=ANYBODY ) {
		$c_group_id	= db_prepare_int( $p_group_id );

		# Optimization when access_level is NOBODY
		if ( NOBODY == $p_access_level ) {
			return array();
		}

		$t_user_table = config_get( 'mantis_user_table' );
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$t_on = ON;

		$t_access_level = $p_access_level;

		$t_access_clause = '';

		if ( $t_access_level > 0 ) {
			$t_access_clause = ' AND access_level >= ' . $t_access_level;
		}

		$t_users = array();

		// Get the project overrides
		$query = "SELECT u.id, u.username
				FROM $t_group_user_list_table l, $t_user_table u
				WHERE l.user_id = u.id
				  AND u.enabled = $t_on
				  AND l.group_id = $c_group_id
				ORDER BY u.username";

		$result = db_query( $query );
		$t_row_count = db_num_rows( $result );
		for ( $i=0 ; $i < $t_row_count ; $i++ ) {
			$row = db_fetch_array( $result );
			
			if ( $f_access_level >= $p_access_level ) {
				$t_users[$row['id']] = $row;
				array_push( $t_users, $row['id'] );
			}
		}

		return $t_users;
	}
	

	#===================================
	# Data Modification
	#===================================
	
	# --------------------
	# add user to a group
	function group_add_user( $p_group_id, $p_user_id ) {
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$c_group_id	= db_prepare_int( $p_group_id );
		$c_user_id	= db_prepare_int( $p_user_id );

		$query = "INSERT
				  INTO $t_group_user_list_table
				    ( group_id, user_id )
				  VALUES
				    ( '$c_group_id', '$c_user_id' )";
		
		db_query( $query );

		# db_query errors on failure so:
		return true;
	}
	
	# --------------------
	# remove user from group
	function group_remove_user( $p_group_id, $p_user_id ) {
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$c_group_id	= db_prepare_int( $p_group_id );
		$c_user_id	= db_prepare_int( $p_user_id );

		$query = "DELETE FROM $t_group_user_list_table
				  WHERE group_id='$c_group_id' AND
						user_id='$c_user_id'";

		db_query( $query );

		# db_query errors on failure so:
		return true;
	}

	# --------------------
	# delete all users from the group user list for a given group
	# this is useful when deleting a group
	function group_remove_all_users( $p_group_id ) {
		$t_group_user_list_table = config_get( 'mantis_group_user_list_table' );

		$c_group_id	= db_prepare_int( $p_group_id );

		$query = "DELETE FROM $t_group_user_list_table
				WHERE group_id='$c_group_id'";

		db_query( $query );

		# db_query errors on failure so:
		return true;
	}

	# --------------------
	# Set a group field
	function group_set_field( $p_group_id, $p_field_name, $p_field_value ) {
		$c_group_id	= db_prepare_int( $p_group_id );
		$c_field_name	= db_prepare_string( $p_field_name );
		$c_field_value	= db_prepare_string( $p_field_value );

		group_ensure_unprotected( $p_group_id );

		$t_group_table = config_get( 'mantis_group_table' );

		$query = "UPDATE $t_group_table
				  SET $c_field_name='$c_field_value'
				  WHERE id='$c_group_id'";

		db_query( $query );

		group_clear_cache( $p_group_id );

		#db_query() errors on failure so:
		return true;
	}
	
	# --------------------
	# 
	
	# --------------------
	# Set the user's groupname to the given string after checking that it is valid
	function group_set_name( $p_group_id, $p_groupname ) {
		group_ensure_name_valid( $p_groupname );
		group_ensure_name_unique( $p_groupname );

		return group_set_field( $p_group_id, 'groupname', $p_groupname );
	}
	
			
?>
